@@ -8,7 +8,7 @@
 #' @param peakLabel The list of the labels of the peaks
 #' @param startP The starting point of the segment.
 #' @param endP The ending point of the segment.
-#' @param distanceMethod The distance method for the hierarchial clustering algorithm.
+#' @param distanceMethod The distance method for the hierarchical clustering algorithm.
 #' @param maxShift The maximum number of points for a shift step.
 #' @param acceptLostPeak This is an option for users, TRUE is the default value. If the users believe that all the peaks in the peak list are true positive, change it to FALSE.
 #' 
@@ -50,10 +50,20 @@ hClustAlign <-function(refSpec, tarSpec, peakList, peakLabel, startP, endP,
     minpeakList=min(peakList)[1];
     maxpeakList=max(peakList)[1];
     startCheckP=startP+which.min(tarSpec[startP:(minpeakList-1)])[1]-1;
-    if (is.na(startCheckP)) startCheckP=startP;
+    if (is.na(startCheckP)) {
+        startCheckP=startP;
+    }
+    if(startCheckP < 1){
+        startCheckP <- startP
+    }
     
     endCheckP=maxpeakList+ which.min(tarSpec[(maxpeakList+1):endP])[1];
-    if (is.na(endCheckP)) endCheckP=endP;
+    if (is.na(endCheckP)){
+        endCheckP=endP;
+    } 
+    if(endCheckP > length(tarSpec)){
+        endCheckP <- endP
+    }
     
     if ((endCheckP-startCheckP)<2) {
         return (list(tarSpec=tarSpec,peakList=peakList));
@@ -73,7 +83,7 @@ hClustAlign <-function(refSpec, tarSpec, peakList, peakLabel, startP, endP,
             peakListTarget=which(peakLabel==0);
             peakList[peakListTarget]=peakList[peakListTarget]+adj$stepAdj;
             
-            lostPeaks=which(peakList<=0);
+            lostPeaks <- which(peakList <= 0 | peakList > length(tarSpec))
             if (length(lostPeaks) >0){
                 
                 peakList=peakList[-lostPeaks];
@@ -102,40 +112,68 @@ hClustAlign <-function(refSpec, tarSpec, peakList, peakLabel, startP, endP,
     if (maxsubData1<minsubData2){
         endP1=maxsubData1+which.min(tarSpec[(maxsubData1+1) :(minsubData2-1)])[1];
         if (is.na(endP1)) endP1=maxsubData1;
+        if(endP1 > length(tarSpec)){
+            endP1 <- maxsubData1
+        }
         startP2=endP1+1;
         if (length(unique(subLabel1))>1){
             res=hClustAlign(refSpec,tarSpec,subData1,subLabel1,startP,endP1,
                             distanceMethod=distanceMethod,maxShift=maxShift,
                             acceptLostPeak=acceptLostPeak);
             tarSpec=res$tarSpec;
-            peakList[labelID1]=res$peakList;
+            if(length(labelID1) == length(res$peakList)){
+                peakList[labelID1] <- res$peakList
+            } else{ 
+                res$peakList <- c(res$peakList, rep(res$peakList[1], length(labelID1)- length(res$peakList) ) )
+                peakList[labelID1] <- res$peakList
+            }
         }        
         if (length(unique(subLabel2))>1){
             res=hClustAlign(refSpec,tarSpec,subData2,subLabel2,startP2,endP,
                             distanceMethod=distanceMethod,maxShift=maxShift,
                             acceptLostPeak=acceptLostPeak);
             tarSpec=res$tarSpec;
-            peakList[labelID2]=res$peakList;
+      
+            if(length(labelID2) == length(res$peakList)){
+                peakList[labelID2]=res$peakList
+            } else{ 
+                res$peakList <- c(res$peakList, rep(res$peakList[1], length(labelID2)- length(res$peakList) ) )
+                peakList[labelID2]=res$peakList
+            }
         }
     }else{        
         maxsubData2=max(subData2)[1];
         minsubData1=min(subData1)[1];
         endP2=maxsubData2+which.min(tarSpec[(maxsubData2+1) :(minsubData1-1)])[1];
         if (is.na(endP2)) endP2=maxsubData2;
+        if(endP2 > length(tarSpec)){
+            endP2 <- maxsubData2
+        }
         startP1=endP2+1;
         if (length(unique(subLabel2))>1){
             res=hClustAlign(refSpec,tarSpec,subData2,subLabel2,startP,endP2,
                             distanceMethod=distanceMethod,maxShift=maxShift,
                             acceptLostPeak=acceptLostPeak);
             tarSpec=res$tarSpec;
-            peakList[labelID2]=res$peakList;
+            
+            if(length(labelID2) == length(res$peakList)){
+                peakList[labelID2]=res$peakList
+            } else{ 
+                res$peakList <- c(res$peakList, rep(res$peakList[1], length(labelID2)- length(res$peakList) ) )
+                peakList[labelID2]=res$peakList
+            }
         }    
         if (length(unique(subLabel1))>1){
             res=hClustAlign(refSpec,tarSpec,subData1,subLabel1,startP1,endP,
                             distanceMethod=distanceMethod,maxShift=maxShift,
                             acceptLostPeak=acceptLostPeak);
             tarSpec=res$tarSpec;
-            peakList[labelID1]=res$peakList;
+            if(length(labelID1) == length(res$peakList)){
+                peakList[labelID1] <- res$peakList
+            } else{ 
+                res$peakList <- c(res$peakList, rep(res$peakList[1], length(labelID1)- length(res$peakList) ) )
+                peakList[labelID1] <- res$peakList
+            }
         }        
     }    
     return (list(tarSpec=tarSpec,peakList=peakList));
